! ---------------------------------------------------------------------------
!  Deblur_IRLS.f90 - Created by Alejandro Valenciano
!           Jul 15 2003.
! ---------------------------------------------------------------------------
!       Usage: < in.H  Deblur_IRLS.x filt=filter.H niter=30 eps=.01 d=40 nqiter=3 alpha=0.05> out.H 
!  Parameters: 
! Description: 
! ---------------------------------------------------------------------------
program Deblur_IRLS
use sep
use solver_reg_mod
use scaled_identity
use cgstep_mod
use blur_mod
use gradchain
use igrad1


implicit none
;
integer                     :: i,n,n1,n2, nf1,nf2,stat, niter,d,nq,iq,in
real                        :: d1, d2, o1, o2, eps, alp
real, dimension(:), pointer :: model, data,q,qq
real, dimension(:,:), pointer :: filt1,filt2

call sep_init(SOURCE)

call from_aux( 'filt', 'n1', nf1)
call from_aux( 'filt', 'n2', nf2)

allocate( filt1(nf1,nf2) )
call sep_read( filt1, 'filt')
allocate( filt2(nf1,nf2) )
call sep_read( filt2, 'filt')

call from_history('n1',n1); call from_history('d1',d1); call from_history('o1',o1)
call from_history('n2',n2); call from_history('d2',d2); call from_history('o2',o2)
n=n1*n2

call from_param( 'alpha', alp, 0.05)
call from_param( 'niter', niter, 10)
call from_param( 'eps', eps, 1.0)
call from_param( 'd', d, 40)
call from_param( 'nqiter', nq, 3)

call to_history('n1',n1);   call to_history('d1',d1);   call to_history('o1',o1)
call to_history('n2',n2);   call to_history('d2',d2);   call to_history('o2',o2)
call to_history('n3',nq);   call to_history('d2',1);   call to_history('o2',1)

allocate( data(n), model(n), q(n),qq(n))

call sep_read( data)
model=0.
qq=0.

do iq= 1, nq 
  write(0,*) "iteration Q",iq
  if (iq == 1 ) then 
   q=1.
  else
!    do in= 1, n-1  
!      qq(in)=model(in+1)-model(in)  
!    end do
!    qq(n)=qq(n-1) 
    stat = igrad1_lop( .false.,.false. , model, qq )
    q=1./(sqrt(1+(qq/alp)**2))  
    write(0,*)"minval(model),maxval(model)", minval(model),maxval(model)
    write(0,*)"minval(qq),maxval(qq)", minval(abs(qq)),maxval(abs(qq)),alp
    write(0,*)"minval(q),maxval(q) ", minval(q),maxval(q)
  end if
   
  call blur_init(filt1,filt2,256)
  call gradchain_init( q )
  
  call solver_reg( model, data, blur_op, weighted_grad_lop, cgstep, n, niter, eps, &
                    m0=model, verb=.true. )
  call blur_release
  call sep_write( model )
end do

deallocate( model, data, filt1,filt2,q)

call exit(0)

end program Deblur_IRLS
