! ---------------------------------------------------------------------------
!  Deblur_IRLS.f90 - Created by Alejandro Valenciano
!           Jul 15 2003.
! ---------------------------------------------------------------------------
!       Usage: < in.H  Deblur_IRLS.x filt=filter.H niter=30 eps=.01 d=40 nqiter=3 alpha=0.05> out.H 
!  Parameters: 
! Description: 
! ---------------------------------------------------------------------------
program Deblur_IRLS_2fit_2d
use sep
use solver_reg_mod
use scaled_identity
use cgstep_mod
use blur_mod
use gradchain_2fit_mod
use fit2_mod
use dottest

implicit none
;
integer                     :: i,n,n1,n2, nf1,nf2, niter,d,nq,iq,in,stat
real                        :: d1, d2, o1, o2, eps, alp
real, dimension(:), pointer :: model, data,q,qx,qz,qqx,qqz;
real, dimension(:,:), pointer :: filt1,filt2
real,    dimension( 2)               :: dot1,dot2

call sep_init(SOURCE)

call from_aux( 'filt', 'n1', nf1)
call from_aux( 'filt', 'n2', nf2)

allocate( filt1(nf1,nf2) )
call sep_read( filt1, 'filt')
allocate( filt2(nf1,nf2) )
call sep_read( filt2, 'filt')

call from_history('n1',n1); call from_history('d1',d1); call from_history('o1',o1)
call from_history('n2',n2); call from_history('d2',d2); call from_history('o2',o2)
n=n1*n2

call from_param( 'alpha', alp, 0.05)
call from_param( 'niter', niter, 10)
call from_param( 'eps', eps, 1.0)
call from_param( 'd', d, 40)
call from_param( 'nqiter', nq, 3)

call to_history('n1',n1);   call to_history('d1',d1);   call to_history('o1',o1)
call to_history('n2',n2);   call to_history('d2',d2);   call to_history('o2',o2)
call to_history('n3',nq);   call to_history('d2',1);   call to_history('o2',1)

if( exist_file("qx") ) then 
  call to_history ("n1",n1,"qx");   call to_history('d1',d1,"qx");   call to_history('o1',o1,"qx") 
  call to_history ("n2",n2,"qx");   call to_history('d2',d2,"qx");   call to_history('o2',o2,"qx")
  call to_history ("n3",nq,"qx");   call to_history('d2',1 ,"qx");   call to_history('o2',1 ,"qx")
end if
if( exist_file("qz") ) then 
  call to_history ("n1",n1,"qz");   call to_history('d1',d1,"qz");   call to_history('o1',o1,"qz") 
  call to_history ("n2",n2,"qz");   call to_history('d2',d2,"qz");   call to_history('o2',o2,"qz")
  call to_history ("n3",nq,"qz");   call to_history('d2',1 ,"qz");   call to_history('o2',1 ,"qz")
end if

allocate( data(n), model(n),q(n), qx(n),qz(n),qqx(n),qqz(n))

call sep_read( data)
model=0.
qqx=0.
qqz=0.

call blur_init(filt1,filt2,256)
call gradchain2d_init( n1, n2, 1.)

do iq= 1, nq 
  write(0,*) "iteration Q",iq
  if (iq == 1 ) then 
   qx=1.
   qz=1.
  else
    stat = xxgrad1_lop(.false.,.false.,model,qqx) 
    stat = ttgrad1_lop(.false.,.false.,model,qqz) 
    qx=1./(sqrt(1+(qqx/alp)**2))  
    qz=1./(sqrt(1+(qqz/alp)**2))
    q=qx+qz  
    write(0,*)"minval(model),maxval(model)", minval(model),maxval(model)
    write(0,*)"minval(qqx),maxval(qqx)", minval(abs(qqx)),maxval(abs(qqx)),alp
    write(0,*)"minval(qqz),maxval(qqz)", minval(abs(qqz)),maxval(abs(qqz)),alp
    write(0,*)"minval(qx),maxval(qx) ", minval(qx),maxval(qx)
    write(0,*)"minval(qz),maxval(qz) ", minval(qz),maxval(qz)
  end if
  call blur_init(filt1,filt2,256)
  call gradchain2d_init( n1, n2, 1.)
  call gradchain_init( qx,qz ) 
  call init_array(n)
  call dot_test(my_array,n,2*n,dot1,dot2)
  call solver_reg( model, data, blur_op, my_array, cgstep, 2*n, niter, eps, &
                    m0=model, verb=.true. )
  call blur_release
  call sep_write( model )
  if( exist_file("qx") ) call sep_write( qx, "qx")
  if( exist_file("qz") ) call sep_write( qz, "qz")
end do

deallocate(model,data,filt1,filt2,q,qx,qz,qqx,qqz)

call exit(0)

end program Deblur_IRLS_2fit_2d
