! ---------------------------------------------------------------------------
!  Deblur_IRLS.f90 - Created by Alejandro Valenciano
!           Jul 15 2003.
! ---------------------------------------------------------------------------
!       Usage: < in.H  Deblur_IRLS.x filt=filter.H niter=30 eps=.01 d=40 nqiter=3 alpha=0.05> out.H 
!  Parameters: 
! Description: 
! ---------------------------------------------------------------------------
program Deblur_helix_IRLS

use sep
use solver_reg_mod
use scaled_identity
use cgstep_mod
use blur_mod
use helderivchain
use helderiv

implicit none

integer                       :: i,n,n1,n2, nf1,nf2,stat, niter,d,nq,iq,in,na
real                          :: d1, d2, o1, o2, eps, alp, eps1
real, dimension(:)  , pointer :: model, data,q,qq
real, dimension(:,:), pointer :: filt1,filt2

call sep_init(SOURCE)

call from_aux( 'filt', 'n1', nf1)
call from_aux( 'filt', 'n2', nf2)

allocate( filt1(nf1,nf2) )
call sep_read( filt1, 'filt')
allocate( filt2(nf1,nf2) )
call sep_read( filt2, 'filt')

call from_history('n1',n1); call from_history('d1',d1); call from_history('o1',o1)
call from_history('n2',n2); call from_history('d2',d2); call from_history('o2',o2)
n=n1*n2

call from_param( 'alpha', alp, 0.05)
call from_param( 'niter', niter, 10)
call from_param( 'eps', eps, 1.0)
call from_param( 'd', d, 40)
call from_param( 'nqiter', nq, 3)
call from_param( 'na', na, 6)
call from_param( 'eps1', eps1, 0.5)

call to_history('n1',n1);   call to_history('d1',d1);   call to_history('o1',o1)
call to_history('n2',n2);   call to_history('d2',d2);   call to_history('o2',o2)
call to_history('n3',nq);   call to_history('d2',1);   call to_history('o2',1)

if( exist_file("q") ) then 
  call to_history ("n1",n1,"q");   call to_history('d1',d1,"q");   call to_history('o1',o1,"q") 
  call to_history ("n2",n2,"q");   call to_history('d2',d2,"q");   call to_history('o2',o2,"q")
  call to_history ("n3",nq,"q");   call to_history('d2',1 ,"q");   call to_history('o2',1 ,"q")
end if
if( exist_file("qq") ) then 
  call to_history ("n1",n1,"qq");   call to_history('d1',d1,"qq");   call to_history('o1',o1,"qq") 
  call to_history ("n2",n2,"qq");   call to_history('d2',d2,"qq");   call to_history('o2',o2,"qq")
  call to_history ("n3",nq,"qq");   call to_history('d2',1 ,"qq");   call to_history('o2',1 ,"qq")
end if

allocate( data(n), model(n), q(n), qq(n))
call sep_read( data)
model=0.
qq=0.

do iq= 1, nq 
  write(0,*) "iteration Q",iq
  if (iq == 1 ) then 
   q=1
   call helderivchain_init( q ,n1, na, eps1)
   call blur_init(filt1,filt2,256)
   call solver_reg( model, data, blur_op,weighted_helderiv_lop, cgstep, n, niter, 0., &
                    m0=model, verb=.true. )
   call blur_release
   call sep_write( model )
   if( exist_file("q") ) call sep_write( q, "q") 
   if( exist_file("qq") ) call sep_write( abs(qq), "qq") 
   call cgstep_close
  else 
   call helderiv_init (n1, na, eps1)
   stat = helderiv_lop (.false.,.false., model, qq)  
   q=1./(sqrt(1.+(qq/alp)**2)) 
   q=abs(qq) !prueba
   write(0,*)"minval(model),maxval(model)", minval(model),maxval(model)
   write(0,*)"minval(qq),maxval(qq)", minval(abs(qq)),maxval(abs(qq)),alp
   write(0,*)"minval(q),maxval(q) ", minval(q),maxval(q)
   call blur_init(filt1,filt2,256)
   call helderivchain_init( q ,n1, na, eps1)
  
   call solver_reg( model, data, blur_op, weighted_helderiv_lop, cgstep, n, niter, eps, &
                     m0=model, verb=.true. )
   call helderiv_close ()
   call blur_release
   call sep_write( model )
   if( exist_file("q") ) call sep_write( q, "q")
   if( exist_file("qq") ) call sep_write( abs(qq), "qq")
   call cgstep_close
  end if
end do

deallocate( model, data, filt1,filt2,q,qq)

call exit(0);

end program Deblur_helix_IRLS
